@@ -15,10 +15,10 @@
 # specific language governing permissions and limitations
 # under the License.
 import uuid
+import queue
 from concurrent.futures import Future
-from six.moves import queue
 
-from gremlin_python.driver import resultset
+from gremlin_python.driver import resultset, useragent
 
 __author__ = 'David M. Brown (davebshow@gmail.com)'
 
@@ -26,7 +26,7 @@ __author__ = 'David M. Brown (davebshow@gmail.com)'
 class Connection:
 
     def __init__(self, url, traversal_source, protocol, transport_factory,
-                 executor, pool, headers=None):
+                 executor, pool, headers=None, enable_user_agent_on_connect=True):
         self._url = url
         self._headers = headers
         self._traversal_source = traversal_source
@@ -37,6 +37,11 @@ class Connection:
         self._pool = pool
         self._results = {}
         self._inited = False
+        self._enable_user_agent_on_connect = enable_user_agent_on_connect
+        if self._enable_user_agent_on_connect:
+            if self._headers is None:
+                self._headers = dict()
+            self._headers[useragent.userAgentHeader] = useragent.userAgent
 
     def connect(self):
         if self._transport:
@@ -53,7 +58,11 @@ class Connection:
     def write(self, request_message):
         if not self._inited:
             self.connect()
-        request_id = str(uuid.uuid4())
+        if request_message.args.get("requestId"):
+            request_id = str(request_message.args.get("requestId"))
+            uuid.UUID(request_id)  # Checks for proper UUID or else server will return an error.
+        else:
+            request_id = str(uuid.uuid4())
         result_set = resultset.ResultSet(queue.Queue(), request_id)
         self._results[request_id] = result_set
         # Create write task
